angular.module('ui.rCalendar', []).constant('calendarConfig', {
  formatDay: 'd',
  formatDayHeader: 'EEE',
  formatDayTitle: 'MMMM dd, yyyy',
  formatWeekTitle: 'MMMM yyyy, Week w',
  formatMonthTitle: 'MMMM yyyy',
  calendarMode: 'month',
  showWeeks: false,
  showEventDetail: true,
  startingDay: 0,
  eventSource: null,
  queryMode: 'local'
}).controller('CalendarController', ['$scope', '$attrs', '$parse', '$interpolate', '$log', 'dateFilter', 'calendarConfig','appntService','treatmentService','$filter', function($scope, $attrs, $parse, $interpolate, $log, dateFilter, calendarConfig, appntService,treatmentService,$filter) {
  var self = this,
    ngModelCtrl = {
      $setViewValue: angular.noop
    }; // nullModelCtrl;
  // Configuration attributes
  angular.forEach(['formatDay', 'formatDayHeader', 'formatDayTitle', 'formatWeekTitle', 'formatMonthTitle', 'showWeeks', 'showEventDetail', 'startingDay', 'eventSource', 'queryMode'], function(key, index) {
    self[key] = angular.isDefined($attrs[key]) ? (index < 5 ? $interpolate($attrs[key])($scope.$parent) : $scope.$parent.$eval($attrs[key])) : calendarConfig[key];
  });
  $scope.$parent.$watch('eventSource', function(value) {
    self.eventSource = value;
    if(self._onDataLoaded) {
      self._onDataLoaded();
    }
  });
  $scope.calendarMode = $scope.calendarMode || calendarConfig.calendarMode;
  if(angular.isDefined($attrs.initDate)) {
    self.currentCalendarDate = $scope.$parent.$eval($attrs.initDate);
  }
  if(!self.currentCalendarDate) {
    self.currentCalendarDate = new Date();
    if($attrs.ngModel && !$scope.$parent.$eval($attrs.ngModel)) {
      $parse($attrs.ngModel).assign($scope.$parent, self.currentCalendarDate);
    }
  }
  self.init = function(ngModelCtrl_) {
    ngModelCtrl = ngModelCtrl_;
    ngModelCtrl.$render = function() {
      self.render();
    };
  };
  self.render = function() {
    if(ngModelCtrl.$modelValue) {
      var date = new Date(ngModelCtrl.$modelValue),
        isValid = !isNaN(date);
      if(isValid) {
        this.currentCalendarDate = date;
      } else {
        $log.error('"ng-model" value must be a Date object, a number of milliseconds since 01.01.1970 or a string representing an RFC2822 or ISO 8601 date.');
      }
      ngModelCtrl.$setValidity('date', isValid);
    }
    this.refreshView();
  };
  self.refreshView = function() {
    if(this.mode) {
      this.range = this._getRange(this.currentCalendarDate);
      this._refreshView();
      this.rangeChanged();
    }
  };
    
  // Split array into smaller arrays
  self.split = function(arr, size) {
    var arrays = [];
    while(arr.length > 0) {
      arrays.push(arr.splice(0, size));
    }
    return arrays;
  };
  $scope.move = function(direction) {
    $scope.$parent.listItems = [];
    var step = self.mode.step,
      currentCalendarDate = self.currentCalendarDate,
      year = currentCalendarDate.getFullYear() + direction * (step.years || 0),
      month = currentCalendarDate.getMonth() + direction * (step.months || 0),
      date = currentCalendarDate.getDate() + direction * (step.days || 0),
      firstDayInNextMonth;
    currentCalendarDate.setFullYear(year, month, date);
    if($scope.calendarMode === 'month') {
      firstDayInNextMonth = new Date(year, month + 1, 1);
      if(firstDayInNextMonth.getTime() <= currentCalendarDate.getTime()) {
        self.currentCalendarDate = new Date(firstDayInNextMonth - 24 * 60 * 60 * 1000);
      }
    }
    ngModelCtrl.$setViewValue(self.currentCalendarDate);
    self.refreshView();
  };
  self.rangeChanged = function() {
    if(self.queryMode === 'local') {
      if(self.eventSource && self._onDataLoaded) {
        self._onDataLoaded();
      }
    } else if(self.queryMode === 'remote') {
      if($scope.rangeChanged) {
        $scope.rangeChanged({
          startTime: this.range.startTime,
          endTime: this.range.endTime
        });
      }
    }
  };

  function overlap(event1, event2) {
    if(event1.endIndex <= event2.startIndex || event2.endIndex <= event1.startIndex) {
      return false;
    }
    return true;
  }

  function calculatePosition(events) {
    var i,
      j,
      len = events.length,
      maxColumn = 0,
      col,
      isForbidden = new Array(len);
    for(i = 0; i < len; i += 1) {
      for(col = 0; col < maxColumn; col += 1) {
        isForbidden[col] = false;
      }
      for(j = 0; j < i; j += 1) {
        if(overlap(events[i], events[j])) {
          isForbidden[events[j].position] = true;
        }
      }
      for(col = 0; col < maxColumn; col += 1) {
        if(!isForbidden[col]) {
          break;
        }
      }
      if(col < maxColumn) {
        events[i].position = col;
      } else {
        events[i].position = maxColumn++;
      }
    }
  }

  function calculateWidth(orderedEvents) {
    var cells = new Array(24),
      event,
      index,
      i;
    //sort by position in descending order, the right most columns should be calculated first
    orderedEvents.sort(function(eventA, eventB) {
      return eventB.position - eventA.position;
    });
    for(i = 0; i < 24; i += 1) {
      cells[i] = {
        calculated: false,
        events: []
      };
    }
    var len = orderedEvents.length;
    for(i = 0; i < len; i += 1) {
      event = orderedEvents[i];
      index = event.startIndex;
      while(index < event.endIndex) {
        cells[index].events.push(event);
        index += 1;
      }
    }
    i = 0;
    while(i < len) {
      event = orderedEvents[i];
      if(!event.overlapNumber) {
        var overlapNumber = event.position + 1;
        event.overlapNumber = overlapNumber;
        var eventQueue = [event];
        while((event = eventQueue.shift())) {
          index = event.startIndex;
          while(index < event.endIndex) {
            if(!cells[index].calculated) {
              cells[index].calculated = true;
              angular.forEach(cells[index].events, function(e) {
                if(!e.overlapNumber) {
                  e.overlapNumber = overlapNumber;
                  eventQueue.push(e);
                }
              });
            }
            index += 1;
          }
        }
      }
      i += 1;
    }
  }
  self.placeEvents = function(orderedEvents) {
    alert(1);
    calculatePosition(orderedEvents);
    calculateWidth(orderedEvents);
  };
  
  self.updateListByDate = function(date) {
        try {
            $scope.$parent.listItems = appntService.selectAppointmentsOnDate(date);
            //$scope.$parent.listItems = appntService.selectAppointmentsOnDate(date).concat(treatmentService.calendarTreatmentArray[$filter('date')(date, "dd MM yyyy")]);
            $scope.$parent.treatmentItems = treatmentService.treatmentOnDate(date);
        } catch(error) {
            $scope.$parent.listItems = [];
            $scope.$parent.treatmentItems = [];
        }
        
  };
  
  self.placeAllDayEvents = function(orderedEvents) {
    calculatePosition(orderedEvents);
  };
}]).directive('calendar', function() {
  return {
    restrict: 'EA',
    replace: true,
    templateUrl: 'templates/rcalendar/calendar.html',
    scope: {
      calendarMode: '=',
      rangeChanged: '&',
      eventSelected: '&',
      timeSelected: '&'
    },
    require: ['calendar', '?^ngModel'],
    controller: 'CalendarController',
    link: function(scope, element, attrs, ctrls) {
      var calendarCtrl = ctrls[0],
        ngModelCtrl = ctrls[1];
      if(ngModelCtrl) {
        calendarCtrl.init(ngModelCtrl);
      }
    }
  };
}).directive('monthview', ['dateFilter','appntService','treatmentService','$filter', function(dateFilter,appntService,treatmentService,$filter) {
  return {
    restrict: 'EA',
    replace: true,
    templateUrl: 'templates/rcalendar/month.html',
    require: ['^calendar', '?^ngModel'],
    link: function(scope, element, attrs, ctrls) {
      var ctrl = ctrls[0],
        ngModelCtrl = ctrls[1];
      scope.showWeeks = ctrl.showWeeks;
      scope.showEventDetail = ctrl.showEventDetail;
      ctrl.mode = {
        step: {
          months: 1
        }
      };

      function getDates(startDate, n) {
        var hasSixWeeks = true;
        var tempDate = new Date(startDate);
        var currentMonth = (startDate.getMonth() + (tempDate.getDate() !== 1 ? 1 : 0)) % 12;
        tempDate.setDate(tempDate.getDate()+35);
        var nextMonth = tempDate.getMonth();
          
        if(currentMonth != nextMonth){
            hasSixWeeks = false;
            n = n-7;
        }
          
        var dates = new Array(n),
          current = new Date(startDate),
          i = 0;
        current.setHours(12); // Prevent repeated dates because of timezone bug
        while(i < n) {
          dates[i++] = new Date(current);
          current.setDate(current.getDate() + 1);
        }
        return dates;
      }
      scope.isAppointmentExist = function(eventObj) {
        if(angular.isUndefined(eventObj)) {
          return false;
        }
        var keepGoing = true;
        var found = false;
        angular.forEach(eventObj, function(record) {
          if(keepGoing) {
            if(record.eventType === 'Appointments') {
              found = true;
              keepGoing = false;
            }
          }
        });
        return found;
      }
      scope.isOthrEvtsExist = function(eventObj) {
        if(angular.isUndefined(eventObj)) {
          return false;
        }
        var keepGoing = true;
        var found = false;
        angular.forEach(eventObj, function(record) {
          if(keepGoing) {
            if(record.eventType === 'others') {
              found = true;
              keepGoing = false;
            }
          }
        });
        return found;
      }
      scope.isEventExist = function(dateObj){
          var date = $filter('date')(dateObj, "dd MM yyyy")
          var number = scope.monthAppntList[date];
          if(number>0){
              return true;
          }else{
            return false;
          }
      }
      scope.isReminderExist = function(dateObj){
          var date = $filter('date')(dateObj, "dd MM yyyy")
          try{
            if(scope.monthTreatmentList[date]){
              return true;
            } else {
              return false;
            }
          }catch(e){
           return false; 
          }
      }
      scope.select = function(selectedDate) {
        var currentCalendarDate = ctrl.currentCalendarDate;
        var currentMonth = currentCalendarDate.getMonth();
          
        if(selectedDate.getMonth()!==currentMonth){
          return;
        }
        var rows = scope.rows;
        if(rows) {
          var currentYear = currentCalendarDate.getFullYear();
          var selectedMonth = selectedDate.getMonth();
          var selectedYear = selectedDate.getFullYear();
          var direction = 0;
          if(currentYear === selectedYear) {
            if(currentMonth !== selectedMonth) {
              direction = currentMonth < selectedMonth ? 1 : -1;
            }
          } else {
            direction = currentYear < selectedYear ? 1 : -1;
          }
          ctrl.currentCalendarDate = selectedDate;
          if(ngModelCtrl) {
            ngModelCtrl.$setViewValue(selectedDate);
          }
          if(direction === 0) {
            for(var row = 0; row < rows.length; row += 1) {
              for(var date = 0; date < 7; date += 1) {
                var selected = ctrl.compare(selectedDate, rows[row][date].date) === 0;
                rows[row][date].selected = selected;
                if(selected) {
                  scope.selectedDate = rows[row][date];
                }
              }
            }
          } 
          if(scope.timeSelected) {
            scope.timeSelected({
              selectedTime: selectedDate
            });
          }
            
            ctrl.updateListByDate(selectedDate);
        }
          
//          $scope.listItems = [
//            {type:"EVENTXYZ",title: "Event 1x"+selectedDate, description:"asasasas"},
//            {type:"REMAINDERXYZ",title: "Event 2y"+selectedDate, description:"dfdfdfdfdf"},
//            {type:"REMAINDERXYZ",title: "Event 3z"+selectedDate, description:"ghghghghghghgh"}
//          ];
          
//          if (!scope.$$phase) {
//                scope.$apply();
//            }
          
      };
      ctrl._refreshView = function() {
        var startDate = ctrl.range.startTime,
          date = startDate.getDate(),
          month = (startDate.getMonth() + (date !== 1 ? 1 : 0)) % 12,
          year = startDate.getFullYear() + (date !== 1 && month === 0 ? 1 : 0);
        var days = getDates(startDate, 42);
        for(var i = 0; i < days.length ; i++) {
          days[i] = angular.extend(createDateObject(days[i], ctrl.formatDay), {
            secondary: days[i].getMonth() !== month
          });
        }
          
          var currentLocaleData = moment.localeData();
          var frLocaleData = moment.localeData('fr');
          scope.labels = frLocaleData._weekdaysShortCapitialize;
          //new Array(7);
          //        for(var j = 0; j < 7; j++) {
          //          scope.labels[j] = dateFilter(days[j].date, ctrl.formatDayHeader);
          //        }
        var headerDate = new Date(year, month, 1);
        scope.$parent.title = frLocaleData._months[month].charAt(0).toUpperCase() + frLocaleData._months[month].slice(1) + " " +year;//dateFilter(headerDate, ctrl.formatMonthTitle);
        scope.rows = ctrl.split(days, 7);
        if(scope.showWeeks) {
          scope.weekNumbers = [];
          var weekNumber = getISO8601WeekNumber(scope.rows[0][0].date),
            numWeeks = scope.rows.length,
            len = 0;
          while(len < numWeeks) {
            len = scope.weekNumbers.push(weekNumber);
            weekNumber += 1;
          }
        }
        ctrl.updateListByDate(ctrl.currentCalendarDate);
      };

      function createDateObject(date, format) {
        return {
          date: date,
          label: dateFilter(date, format),
          selected: ctrl.compare(date, ctrl.currentCalendarDate) === 0,
          current: ctrl.compare(date, new Date()) === 0
        };
      }
      ctrl._onDataLoaded = function() {
        var eventSource = ctrl.eventSource,
          len = eventSource ? eventSource.length : 0,
          startTime = ctrl.range.startTime,
          endTime = ctrl.range.endTime,
          timeZoneOffset = -new Date().getTimezoneOffset(),
          utcStartTime = new Date(startTime.getTime() + timeZoneOffset * 60 * 1000),
          utcEndTime = new Date(endTime.getTime() + timeZoneOffset * 60 * 1000),
          rows = scope.rows,
          oneDay = 24 * 3600 * 1000,
          eps = 0.001;
        //alert(JSON.stringify(eventSource));
        for(var i = 0; i < len; i += 1) {
          var event = eventSource[i];
          var eventStartTime = new Date(event.startTime);
          var eventEndTime = new Date(event.endTime);
          var st;
          var et;
          if(event.allDay) {
            if(eventEndTime <= utcStartTime || eventStartTime >= utcEndTime) {
              continue;
            } else {
              st = utcStartTime;
              et = utcEndTime;
            }
          } else {
            if(eventEndTime <= startTime || eventStartTime >= endTime) {
              continue;
            } else {
              st = startTime;
              et = endTime;
            }
          }
          var timeDifferenceStart;
          if(eventStartTime <= st) {
            timeDifferenceStart = 0;
          } else {
            timeDifferenceStart = (eventStartTime - st) / oneDay;
          }
          var timeDifferenceEnd;
          if(eventEndTime >= et) {
            timeDifferenceEnd = (et - st) / oneDay;
          } else {
            timeDifferenceEnd = (eventEndTime - st) / oneDay;
          }
          var index = Math.floor(timeDifferenceStart);
          var eventSet;
          while(index < timeDifferenceEnd - eps) {
            var rowIndex = Math.floor(index / 7);
            var dayIndex = Math.floor(index % 7);
            rows[rowIndex][dayIndex].hasEvent = true;
            eventSet = rows[rowIndex][dayIndex].events;
            if(eventSet) {
              eventSet.push(event);
            } else {
              eventSet = [];
              eventSet.push(event);
              rows[rowIndex][dayIndex].events = eventSet;
            }
            index += 1;
          }
        }
        var findSelected = false;
        for(var row = 0; row < 6; row += 1) {
          for(var date = 0; date < 7; date += 1) {
            if(rows[row][date].selected) {
              scope.selectedDate = rows[row][date];
              findSelected = true;
              break;
            }
          }
          if(findSelected) {
            break;
          }
        }
      };
      ctrl.compare = function(date1, date2) {
        return(new Date(date1.getFullYear(), date1.getMonth(), date1.getDate()) - new Date(date2.getFullYear(), date2.getMonth(), date2.getDate()));
      };
        
      ctrl._getRange = function getRange(currentDate) {
        var year = currentDate.getFullYear(),
          month = currentDate.getMonth(),
          firstDayOfMonth = new Date(year, month, 1),
          difference = ctrl.startingDay - firstDayOfMonth.getDay(),
          numDisplayedFromPreviousMonth = (difference > 0) ? 7 - difference : -difference,
          startDate = new Date(firstDayOfMonth),
          endDate;
          endDayOfMonth = new Date(year, month+1, 0,23,59,59,999);
          try{
            scope.monthAppntList = appntService.countAppointmentsByDateFromToDate(firstDayOfMonth,endDayOfMonth);
            } catch(error){
              scope.monthAppntList = [];
            }
          try{
            scope.monthTreatmentList = treatmentService.countTreatmentsByDateFromToDate(firstDayOfMonth,endDayOfMonth);
          } catch(error){
              scope.monthTreatmentList = [];
          }
          
        if(numDisplayedFromPreviousMonth > 0) {
          startDate.setDate(-numDisplayedFromPreviousMonth + 1);
        }
        endDate = new Date(startDate);
        endDate.setDate(endDate.getDate() + 42);
        return {
          startTime: startDate,
          endTime: endDate
        };
      };

      function getISO8601WeekNumber(date) {
        var checkDate = new Date(date);
        checkDate.setDate(checkDate.getDate() + 4 - (checkDate.getDay() || 7)); // Thursday
        var time = checkDate.getTime();
        checkDate.setMonth(0); // Compare with Jan 1
        checkDate.setDate(1);
        return Math.floor(Math.round((time - checkDate) / 86400000) / 7) + 1;
      }
      ctrl.refreshView();
    }
  };
}]);